  # schema.rb
require 'jungle_path/db_model'
require 'jungle_path/schema'

module Schema
  # schema meta data:

  def self.version
    # Specify database version number.
    # This version number will be used to apply database
    # migrations.
    # If the configatron.db.name does not exist, then
    # this schema is applied directly and the appropriate
    # version number is recorded in the database.
    # Version of nil will run all available migrations
    #1
    nil
  end

  def self.migrations_path
    ::File.expand_path('../ztools/db/migrations',::File.dirname(__FILE__))
  end

  # base table:

  class Base < DBModel::Table
	end

  # Used for database versioning with Sequel migrations:
  class SchemaInfo < Schema::Base
    self.description = "Used by sequel migration."
    define(
      [:version, :integer, :primary_key]
    )
  end

  # application tables examples:

  class Answer < Schema::Base
    self.description = ""
    define(
      [:id, :primary_key],
      [:question_id, :foreign_key, :question],
      [:description, :string],
      [:is_correct, :boolean],
      [:audit_user]
    )
  end

  class Practice < Schema::Base
    self.description = ""
    define(
      [:id, :primary_key],
      [:user_id, :foreign_key, :user],
      [:notes, :string],
      [:random_question_order, :boolean],
      [:random_answer_order, :boolean],
      [:audit_user]
    )
  end

  class PracticeQuiz < Schema::Base
    self.description = ""
    define(
      [:practice_id, :foreign_key, :practice, :primary_key],
      [:quiz_id, :foreign_key, :quiz, :primary_key],
      [:audit_user]
    )
    def self.plural_table_name
      "practice_quizzes"
    end
  end

  class PracticeQuestion < Schema::Base
    self.description = ""
    define(
      [:practice_id, :foreign_key, :practice, :primary_key],
      [:question_id, :foreign_key, :question, :primary_key],
      [:note, :string],
      [:audit_user]
    )
  end

  class PracticeAnswer < Schema::Base
    self.description = ""
    define(
      [:practice_id, :foreign_key, :practice, :primary_key],
      [:answer_id, :foreign_key, :answer, :primary_key],
      [:label, :string],
      [:selected, :boolean],
      [:audit_user]
    )
  end

  class Question < Schema::Base
    self.description = ""
    define(
      [:id, :primary_key],
      [:quiz_id, :foreign_key, :quiz],
      [:name, :string],
      [:description, :string],
      [:audit_user]
    )
  end

  class Quiz < Schema::Base
    self.description = ""
    define(
      [:id, :primary_key],
      [:name, :string],
      [:description, :string],
      [:private, :boolean, :default, true],
      [:audit_user]
    )
    def self.plural_table_name
      "quizzes"
    end
  end

  class Log < Schema::Base
    define(
      [:id, :primary_key],
      [:set_id, :integer],
      [:name, :string],
      [:type, :string],
      [:item, :string, :secure],
      [:timestamp, :timestamp]
    )
  end


  class Role < Schema::Base
    self.description = "User authorization: Roles for Users to assume in system."
    define(
      [:id, :primary_key],
      [:name, :string, :unique_index, [:name], :not_null],
      [:description, :string],
      [:audit_user]
    )
  end



  class User < Schema::Base
    self.description = "User of system."
    define(
      [:id, :primary_key],
      [:name, :string, :desc, "Your human name! :)"],
      [:first_name, :string, :index],
      [:last_name, :string, :index],
      [:user_name, :string, :unique, :not_null, :alternate_key, :desc, "may use an email address as a user_name"],
      [:email, :string, :index],
      [:phone, :string, :desc, "mobile phone number", :index],
      [:sms_verification_code, :string, :secure, :desc, "phone verification/sign-in code."],
      [:hash, :string, :not_null, :secure, :calculated, :desc, "See password_hash.rb for details."],
      [:key, :string, :unique, :secure, :calculated, :desc, "Used as alternative to user_name/password."],
      [:activation_key, :string, :secure],
      [:active, :boolean],
      [:is_valid, :boolean, :calculated],
      [:password_reset_code, :string, :secure],
      [:audit_user]
    )
  end

  class User
		@auth
		@query_filters
    @password
		attr_accessor :auth, :query_filters, :password
	end


  class UserRole < Schema::Base
    self.description = "Assign authorization roles to users of system."
    define(
      [:user_id, :foreign_key, :user, :primary_key],
      [:role_id, :foreign_key, :role, :primary_key],
      [:audit_user]
    )
  end
end
