require 'securerandom'
require 'sequel'
require 'jungle_path/authentication'
require 'jungle_path/controller'
require 'jungle_path/exceptions'
require 'jungle_path/schema'
require 'jungle_path/sql'

module JunglePath
  module Controller
    class User < Base
      def initialize(current_identity, params, db)
        super(current_identity, params, db, JunglePath::Schema::User)
      end

      def insert(include_secure_columns: false)
        params = self.class.add_audit_parameter_values_for_insert(@params, @current_user, @current_key, @table_class)
        model = @table_class.new params

        password = params[:password]
        message = self.class.validate_password_message(password)
        if message
          self.class.validate_insert(model, message)
          raise JunglePath::Exceptions::InvalidPassword, "#{message}", caller
        end
        # todo: validate password strength, etc. here!
        model.hash = JunglePath::Authentication::PasswordHash.createHash(password)

        self.class.validate_insert(model)
        begin
          result = @db.insert._model(model)
          result = self.class.handle_include_secure_columns_flag(result, include_secure_columns, @table_class)
        rescue Sequel::UniqueConstraintViolation => e
          # already there? update instead...
          #puts "not unique!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
          user_query = JunglePath::Schema::User.new({user_name: params[:user_name]})
          user = @db.select._model_by_any(user_query)
          #puts "user: #{user}."
          params[:id] = user.id if user
          if user
            update
            user = @db.select._model(user)
          else
            raise
          end
        end
      end

      def update
        params = self.class.add_audit_parameter_values_for_update(@params, @current_user, @current_key, @table_class)
        model = @table_class.new params

        password = params[:password]
        if password
          message = self.class.validate_password_message(password)
          if message
            self.class.validate_update(model, message)
            raise JunglePath::Exceptions::InvalidPassword, "#{message}", caller
          end
          # todo: validate password strength, etc. here!
          model.hash = JunglePath::Authentication::PasswordHash.createHash(password)
        end

        self.class.validate_update(model)
        result = @db.update._model(model)
      end

      def self.validate_hash_with_password(hash, password)
        JunglePath::Authentication::PasswordHash.validatePassword(password, hash)
      end

      private
      def self.validate_password_message(password)
        message = nil
        # returns message if password is not a minimum strength...
        message = "Password is either missing or too short." unless password and password.length > 0
      end
    end

    class Key < Base
      def initialize(current_identity, params, db)
        super(current_identity, params, db, JunglePath::Schema::Key)
      end

      def select_by_key(include_secure_columns: false)
        JunglePath::SQL::Key.by_key(@db, @params[:key])
      end

      def select_by_user(include_secure_columns: false)
        JunglePath::SQL::Key.by_user_id(@db, @params[:user_id])
      end

      def select_by_user_by_key_name(include_secure_columns: false)
        JunglePath::SQL::Key.by_user_id_key_name(@db, @params[:user_id], @params[:name])
      end

      def select_default_by_user(include_secure_columns: false)
        JunglePath::SQL::Key.default_by_user_id(@db, @params[:user_id])
      end

      #def select_by_user_by_application(include_secure_columns: false)
      #  @params = Controller::Base.transform(@params, Schema::Role.columns)
      #  SQL::Key.by_user_id_application_id(@db, @params[:user_id], @params[:application_id])
      #end
      #
      #def select_default_by_user_by_application(include_secure_columns: false)
      #  @params = Controller::Base.transform(@params, Schema::Role.columns)
      #  SQL::Key.default_by_user_id_application_id(@db, @params[:user_id], @params[:application_id])
      #end

      def insert(include_secure_columns: false)
        #puts "insert key!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        #already exists?
        array = self.select_by_user_by_key_name
        hash = array.first if array and array.length > 0
        if hash #exists already, do update.
          key = @table_class.new(hash) # get current values which includes primary key (pk is needed for an update).
          key._consume_hash(@params) # overwrite current values with new values
          @params = key.to_hash # put all in @params for update
          update
          key = @db.select._model(key) # return updated object from database.
        else
          @params[:key] = JunglePath::Authentication::Helpers.generate_api_key unless @params[:key] #create a new key_string unless one was passed in.
          super
        end
      end
    end
  end
end
