require 'set'
require 'jungle_path/json'
require 'jungle_path/db_model'

module JunglePath
  module DBModel
		class Table
			@models = {}
			def self.set_columns
				@columns = {}
				@non_audit_columns = {}
				@primary_key_columns = {}
				@alternate_key_columns = {}
				@foreign_key_columns = {}
				@foreign_key_columns_by_table_name = {}
				@references = Set.new
				@secure_columns = {}
				@unique_indexes = []
				@indexes = []
				@table_name = self.name.split('::').last.underscore.to_sym
				@tagged_columns = {}
				@sequence = 0
			end

			def self.columns
				@columns
			end

			def self.columns_alphabetical_order
				@columns.values.clone.sort { |a,b| a.name <=> b.name }
			end

			def self.columns_sequence_order
				@columns.values.clone.sort { |a,b| a.sequence <=> b.sequence }
			end

			def self.non_audit_columns
				@non_audit_columns
			end

			def self.primary_key_columns
				@primary_key_columns
			end

			def self.alternate_key_columns
				@alternate_key_columns
			end

			def self.foreign_key_columns
				@foreign_key_columns
			end

			def self.foreign_key_columns_by_table_name
				@foreign_key_columns_by_table_name
			end

			def self.tagged_columns
				#call like: TableClass.tagged_columns[:my_tag] to return set of columns having given tag.
				@tagged_columns
			end

			def self.references
				@references
			end

			def self.has_reference_to?(table)
				@references.member?(table.table_name)
			end

			def self.has_references?
				!@references.empty?
			end

			def self.has_no_references?
				!self.has_references?
			end

			def self.secure_columns
				@secure_columns
			end

			def self.table_name
        return @override_table_name if @override_table_name
				@table_name
			end

      def self.original_table_name
        @table_name
      end

      def self.override_table_name
        @override_table_name
      end

      def self.override_table_name=(value)
        tmp = nil
        tmp = value.to_sym if value
        @override_table_name = tmp
      end

			def self.plural_table_name
				"#{@table_name}s"
			end

			def self.description
				@description
			end

			def self.description=(value)
				@description = value
			end

      def self.is_view?
	      if @view
	        true
	      else
	        false
	      end
	    end

	    def self.view
	      @view
	    end

	    def self.view=(value)
	      @view = value
	    end

	    def self.view_references
	      if @view
	        @view.references
	      else
	        references
	      end
	    end

	    def self.has_view_reference_to?(table)
	      if @view
	        @view.has_reference_to?(table.table_name)
	      else
	        has_reference_to?(table)
	      end
	    end

	    def self.has_view_references?
	      if @view
	        !@view.references.empty?
	      else
	        has_references?
	      end
	    end

	    def self.has_no_view_references?
	      !self.has_view_references?
	    end

			def self.inherited(sub_class)
				sub_class.set_columns #setup subclass with particular class instance vars.
        @models = {} unless @models
				@models[sub_class.table_name] = sub_class #keep a list of all subclasses.
			end

			def self.models
				@models
			end

			def self.models_alphabetical_order
				@models.values.clone.sort { |a,b| a.table_name <=> b.table_name }
			end

			def self.models_dependent_order
				result = self.model_sort(@models.values.clone)
				# map array of table names into an array of table (model) classes.
				result.map {|key| @models[key]}
			end

      def self.model_sort(list, result=Set.new, level=0)
	      # return array of table names sorted in dependency order (not alphabetical order).
	      #puts "level: #{level}."
	      remaining = []
	      list.each do |value|
	        #puts "\tvalue: #{value.table_name}. references: #{value.references.to_a}."
	        if value.has_no_view_references?
	          #puts "\tadding: #{value.table_name}."
	          result << value.table_name
	        elsif value.view_references.subset? result
	          #puts "\tadding: #{value.table_name}."
	          result << value.table_name
	        else
	          remaining << value
	        end
	      end
	      #puts "\tresult: #{result.length}: #{result.to_a}."
	      #puts "\tremaining: #{remaining.length}: #{remaining}."
	      return [] if level > 10 #something is wrong!!!
	      result = self.model_sort(remaining, result, level + 1) if remaining.length > 0
	      result
	    end

			def self.define(*column_defs)
				set_columns

				column_defs.each do |column_def|
					# one column_def may produce multiple actual columns:
					columns = Column.get_column_def_columns(column_def, @sequence)
					columns.each do |column|
						@sequence = column.sequence + 1
						self.columns[column.name] = column
						self.primary_key_columns[column.name] = column if column.primary_key?
						self.alternate_key_columns[column.name] = column if column.alternate_key?
						self.foreign_key_columns[column.name] = column if column.foreign_key?

						if column.foreign_key?
							hash = self.foreign_key_columns_by_table_name
							sub_hash = hash[column.foreign_key_table_name]
							sub_hash = {} unless sub_hash
							sub_hash[column.name] = column
							hash[column.foreign_key_table_name] = sub_hash
						end

						if column.tags
							column.tags.each do |tag|
								tagged_columns = @tagged_columns[tag] || Set.new
								tagged_columns.add(column)
								@tagged_columns[tag] = tagged_columns
							end
						end

						self.references << column.foreign_key_table_name if column.foreign_key? and column.foreign_key_table_name != self.table_name
						self.secure_columns[column.name] = column if column.secure?
						self.non_audit_columns[column.name] = column unless column.is_audit_column?
						#attr_accessor column.name
						define_method(column.name) do
							@values[column.name]
						end
						define_method("#{column.name}=") do |value|
							if value != @values[column.name]
								@values[column.name] = value
								@modified.add(column.name)
							end
						end
					end
					self.primary_key_columns.values[0].calculated = true if self.primary_key_columns.length == 1
				end

				define_method("_values") do
					@values
				end

				define_method("_modified") do
					@modified
				end

				define_method("_modified_hash") do
					hash = {}
					@modified.each do |key|
						hash[key] = @values[key]
					end
					hash
				end

				define_method("_has_value_hash") do
					hash = {}
					@values.each do |key, value|
						hash[key] = value unless value == nil
					end
					hash
				end

				define_method("_primary_key") do
					primary_key = {}
					_primary_key_columns.keys.each do |key|
						primary_key[key] = @values[key]
					end
					primary_key
				end

				define_method("_alternate_key") do
					alternate_key = {}
					_alternate_key_columns.keys.each do |key|
						alternate_key[key] = @values[key]
					end
					alternate_key
				end

				define_method("_consume_hash") do |hash, mark_as_modified_if_changed=true|
					@values.each do |key, value|
						if hash.has_key?(key) and @values[key] != hash[key]
							@values[key] = hash[key]
							@modified.add(key) if mark_as_modified_if_changed
						end
					end
				end

				define_method("_columns") do
					self.class.columns
				end

				define_method("_primary_key_columns") do
					self.class.primary_key_columns
				end

				define_method("_alternate_key_columns") do
					self.class.alternate_key_columns
				end

				define_method("_foreign_key_columns") do
					self.class.foreign_key_columns
				end

				define_method("_secure_columns") do
					self.class.secure_columns
				end

				define_method("_tagged_columns") do
					self.class.tagged_columns
				end

				define_method("_table_name") do
					self.class.table_name
				end

				define_method("_secure?") do
					@secure
				end

				define_method("_secure=") do |value|
					@secure = value
				end

				define_method("to_hash") do
					#puts "#{_table_name} to_hash."
					return @values.clone if _secure_columns.length == 0 or !_secure?
					hash = {}
					puts "stripping secure columns!"
					@values.each do |key, value|
						hash[key] = value unless _secure_columns.has_key?(key)
					end
					hash
				end

				define_method("to_h") do
					# ruby way :)
					to_hash
				end

				define_method("to_json") do |arg=nil|
					#puts "#{_table_name} to_json."
					#puts "to_json - arg: #{arg.class}" if arg
					json = JunglePath::Json.dump self
					#puts json
					json
				end
			end

			def initialize(hash=nil, mark_as_modified_if_changed=true, transform_params=false)
				@values = {}
				@secure = true
				_columns.keys.each do |key| @values[key] = nil end
				@modified = Set.new # holds hash keys of modified values.
				hash = DBModel::Params.transform(hash, _columns) if transform_params
				_consume_hash(hash, mark_as_modified_if_changed) if hash
			end
		end
	end
end
